package org.netkernel.layer0.nkf;

import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.urii.IEndpoint;
import org.netkernel.urii.IResolution;
import org.netkernel.urii.ISpace;

public interface INKFResolutionContext extends INKFRequestContext
{
	/** Returns the request we are being asked to resolve
	 */
	INKFRequestReadOnly getRequestToResolve();

	/** Delegate resolution into the given space. If the delegate space succesfully resolves
	 * it will set the response. This can be checked with isResponseSet() method.
	 * @param aDelegate The space to attempt delegation to
	 * @throws NKFException thrown if there is a problem
	 */
	void delegateResolution(ISpace aDelegate) throws NKFException;
	
	/** Returns true if this resolve request wants to capture resolution debug 
	 */
	boolean isDebugResolve();
	
	/** Adds a debug string to the resolve debug trace 
	 * @param aDebug
	 */
	void addResolutionDebug(String aDebug);
	
	/** Adds a debug string to the resolve debug trace with addition details
	 * @param aDebug
	 * @param aLevel
	 * @param aEntity
	 */
	void addResolutionDebug(String aDebug, int aLevel, Object aEntity);
	void insertResolutionDebug(String aDebug, int aLevel, Object aEntity);
	void pushResolutionDebug();
	void popResolutionDebug();
	
	
	/** Creates a response for matching on the given endpoint and sets it as THE response
	 * @param aEndpoint the endpoint resolved for this request
	 * @return the response for further manipulation
	 */
	INKFResponse createResolutionResponse(IEndpoint aEndpoint);
	
	/** Creates a response for matching on the given endpoint and sets it as THE response
	 * @param aEndpoint the endpoint resolved for this request
	 * @param aIdentifier the endpoint identifier resolved for this request
	 * @return the response for further manipulation
	 */
	INKFResponse createResolutionResponse(IEndpoint aEndpoint, String aIdentifier);
	
	INKFResponse createResolutionResponse(IEndpoint aEndpoint, String aIdentifier, IRequestScopeLevel aRequestScope);
	
	INKFResponse createResolutionTerminateResponse();
	
	/** Return the previously set resolution response
	 * @return
	 */
	IResolution getResolution();
	
	/** Issue a resolve request. Useful for conditional resolves
	 * @param aSubRequest the request to resolve
	 * @return the IResolution or null if no resolve
	 * @throws NKFException if there is an exception whilst executing sub-request
	 */
	IResolution resolve(INKFRequest aSubRequest) throws NKFException;
	
	/** Performs a request to see if we can validly transrept primary to given representation class
	 * @param aClass
	 * @return
	 * @throws NKFException
	 */
	boolean canSourcePrimaryAs(Class aClass) throws NKFException;
	
}
