/******************************************************************************
 * (c) Copyright 2002-2006, 1060 Research Ltd
 *
 * This Software is licensed to You, the licensee, for use under the terms of
 * the 1060 Public License v1.0. Please read and agree to the 1060 Public
 * License v1.0 [www.1060research.com/license] before using or redistributing
 * this software.
 *
 * In summary the 1060 Public license has the following conditions.
 * A. You may use the Software free of charge provided you agree to the terms
 * laid out in the 1060 Public License v1.0
 * B. You are only permitted to use the Software with components or applications
 * that provide you with OSI Certified Open Source Code [www.opensource.org], or
 * for which licensing has been approved by 1060 Research Limited.
 * You may write your own software for execution by this Software provided any
 * distribution of your software with this Software complies with terms set out
 * in section 2 of the 1060 Public License v1.0
 * C. You may redistribute the Software provided you comply with the terms of
 * the 1060 Public License v1.0 and that no warranty is implied or given.
 * D. If you find you are unable to comply with this license you may seek to
 * obtain an alternative license from 1060 Research Limited by contacting
 * license@1060research.com or by visiting www.1060research.com
 *
 * NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC
 * LICENSE V1.0 FOR DETAILS
 *
 * THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ
 * THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license
 *
 * File:          $RCSfile$
 * Version:       $Name$ $Revision$
 * Last Modified: $Date$
 *****************************************************************************/
package org.netkernel.layer0.nkf.impl;

import org.netkernel.layer0.nkf.*;
import org.netkernel.request.*;
import org.netkernel.request.impl.ExpiryFactory;
import org.netkernel.request.impl.MetaImpl;
import org.netkernel.request.impl.RequestResponseFieldsImpl;
import org.netkernel.request.impl.ResponseImpl;
import java.util.*;
/**
 * Wrapper around the setting of a response from the invocation of an accessor
 * @author  tab
 */
public class NKFResponseImpl implements INKFResponse, INKFResponseReadOnly
{
	private Object mRepresentation;
	private NKFContextImpl mContext;
	private IExpiry mExpiry;
	private IRequestResponseFields mMetaData;
	private IRequestResponseFields mInitialMetaData;
	
	/** Creates a new instance of NKFResponseImpl */
	public NKFResponseImpl(Object aRep, IRequestResponseFields aUserMeta, NKFContextImpl aContext)
	{	mRepresentation=aRep;
		mContext=aContext;
		mExpiry=ExpiryFactory.getDependentExpiry();
		mMetaData=aUserMeta;
		mInitialMetaData=mMetaData;
	}	
	
	/** Creates a new instance of NKFResponseImpl */
	public NKFResponseImpl(Object aRep, NKFContextImpl aContext)
	{	this(aRep, RequestResponseFieldsImpl.EMPTY, aContext);
	}
	
	/** Creates a new instance of NKFResponseImpl */
	public NKFResponseImpl(INKFResponseReadOnly aOther, NKFContextImpl aContext)
	{	mContext=aContext;
		mExpiry=ExpiryFactory.getDependentExpiry();
		if (aOther!=null)
		{	mRepresentation=aOther.getRepresentation();
			if (aOther instanceof NKFResponseReadOnlyImpl)
			{	mMetaData = ((NKFResponseReadOnlyImpl)aOther).getUserMeta();
			}
			else
			{	mMetaData = ((NKFResponseImpl)aOther).getUserMeta();
			}
		}
		else
		{	mMetaData=RequestResponseFieldsImpl.EMPTY;
		}
		mInitialMetaData=mMetaData;
	}
	
	IRequestResponseFields getUserMeta()
	{	return mMetaData;
	}
	
	public IExpiry getExpiry()
	{	return mExpiry;
	}
	
	public IResponse getKernelResponse()
	{	NKFRequestReadOnlyImpl nkfRequest = mContext.getThisRequestOrNull();
		IRequest request;
		if (nkfRequest!=null)
		{	request = nkfRequest.getKernelRequest();
		}
		else
		{	request = null;
		}
		List<IResponse> additionalDependencies = mContext.getAdditionalDependencies();
		if (additionalDependencies!=null)
		{	setHeader(IResponseMeta.KEY_ADDITIONAL_DEPENDENCIES, additionalDependencies);
		}

		MetaImpl meta = new MetaImpl(mExpiry,mMetaData);
		IResponse result = new ResponseImpl(request,mRepresentation,meta);
		return result;
	}
	
	public void setExpiry(int aType)
	{	switch (aType)
		{	case EXPIRY_ALWAYS:
				mExpiry=ExpiryFactory.getAlwaysExpired();
				break;
			case EXPIRY_DEPENDENT:
				mExpiry=ExpiryFactory.getDependentExpiry();
				break;
			case EXPIRY_NEVER:
				mExpiry=ExpiryFactory.getNeverExpired();
				break;
			default:
				throw new IllegalArgumentException();
		}
	}

	public void setExpiry(int aType, long aConstant)
	{	switch (aType)
		{	case EXPIRY_CONSTANT:
			mExpiry=ExpiryFactory.getTimedExpiry(aConstant);
			break;
		case EXPIRY_MIN_CONSTANT_DEPENDENT:
			mExpiry=ExpiryFactory.getMinConstantDependentExpiry(aConstant);
			break;
		case EXPIRY_MAX_CONSTANT_DEPENDENT:
			mExpiry=ExpiryFactory.getMaxConstantDependentExpiry(aConstant);
			break;
		case EXPIRY_POLLED_DEPENDENT:
			mExpiry=ExpiryFactory.getPolledDependentExpiry(aConstant);
			break;
		default:
			throw new IllegalArgumentException();
		}
	}

	public void setExpiry(int aType, INKFExpiryFunction aFunction)
	{	switch (aType)
		{	case EXPIRY_FUNCTION:
			mExpiry=aFunction;
			break;
		case EXPIRY_MIN_FUNCTION_DEPENDENT:
			mExpiry=ExpiryFactory.getMinFunctionDependentExpiry(aFunction);
			break;
		default:
			throw new IllegalArgumentException();
		}
	}

	public void setHeader(String aKey, Object aRepresentation)
	{	if (mMetaData==mInitialMetaData)
		{	mMetaData=new RequestResponseFieldsImpl(mInitialMetaData);
		}
		RequestResponseFieldsImpl meta = (RequestResponseFieldsImpl)mMetaData;
		if (aRepresentation!=null)
		{	meta.put(aKey, aRepresentation);
		}
		else
		{	meta.remove(aKey);
		}
	}

	public void setMimeType(String aType)
	{	setHeader(IResponseMeta.KEY_MIMETYPE,aType);
	}
	
	//////////////////////////////////
	// INKFResponseReadOnly interface
	//////////////////////////////////

	public Object getRepresentation()
	{	return mRepresentation;
	}
	
	public boolean hasHeader(String aKey)
	{	return mMetaData.getValue(aKey)!=null;
	}

	public Object getHeader(String aKey)
	{	return mMetaData.getValue(aKey);
	}

	public Object getHeaderAs(String aKey, Class aRepresentationClass) throws NKFException
	{	
		Object meta = getHeader(aKey);
		if (meta!=null && !aRepresentationClass.isAssignableFrom(meta.getClass()))
		{	meta = mContext.transrept(meta, aRepresentationClass);
		}
		return meta;
	}

	public Iterator getHeaderNames()
	{	int size=mMetaData.size();
		List result = new ArrayList(size);
		for (int i=0; i<size; i++)
		{	result.add(mMetaData.getKey(i));
		}
		return result.iterator();
	}

	public String getMimeType()
	{	String result = (String)getHeader(IResponseMeta.KEY_MIMETYPE);
		if (result==null)
		{	result=MIME_UNKNOWN;
		}
		return result;
	}

	public void setNoCache()
	{	setHeader(IResponseMeta.KEY_NO_CACHE, Boolean.TRUE);
	}
	
	public void setCacheBoost(int aBoost)
	{	int boost = (aBoost>0)?aBoost:0;
		setHeader(IResponseMeta.KEY_CACHE_BOOST, Integer.valueOf(boost));
	}

	public INKFRequest getRequest()
	{	// not valid to ask for request when response hasn't been assigned to request yet
		throw new IllegalArgumentException();
	}

	public boolean isExpired()
	{	return mExpiry.isExpired(System.currentTimeMillis());
	}

	public void setHeaders(IRequestResponseFields aMetadata)
	{	mMetaData=aMetadata;
	}

	public IRequestResponseFields getHeaders()
	{	return mMetaData;
	}

}