package org.netkernel.layer0.representation.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.netkernel.layer0.representation.IHDSInversion;
import org.netkernel.layer0.representation.IHDSNode;
import org.netkernel.layer0.representation.IHDSNodeList;
import org.netkernel.layer0.util.HDSUtils;

public class HDSInversionImpl implements IHDSInversion
{
	private IHDSNode mRoot;
	private Map<Object, List<IHDSNode>> mMap;
	
	public HDSInversionImpl(IHDSNode aRoot)
	{	mRoot=aRoot;
		mMap=new HashMap<Object, List<IHDSNode>>();
		recurseHDS(aRoot);		//Use this rather than HDSUtils in order traversal to avoid NPE with moduleListDoc
	}
	
	private void recurseHDS(IHDSNode aNode)
	{	mapNode(aNode);
		for (IHDSNode child :aNode.getChildren())
		{	recurseHDS(child);
		}
	}
	
	public IHDSNodeList getNodes(Object value)
	{	List<IHDSNode> list=mMap.get(value);
		IHDSNodeList result=null;
		if(list!=null && list.size()>0)
		{	result=new HDSNodeListImpl(list);			
		}
		return result; 
	}
	
	public IHDSNodeList getNodes(IHDSValueMatcher matcher)
	{	Set keys=mMap.keySet();
		ArrayList<IHDSNode> a=new ArrayList();
		for(Iterator it=keys.iterator(); it.hasNext(); )
		{	Object v=it.next();
			if(matcher.matches(v))
			{	List<IHDSNode> nodeList=mMap.get(v);
				for (IHDSNode node : nodeList)
				{	a.add(node);
				}
			}
		}
		IHDSNodeList result=null;
		if(a.size()>0)
		{	result=new HDSNodeListImpl(a);			
		}
		return result; 
	}
	
	public IHDSNodeList getNodesWithAncestor(IHDSNode ancestor, Object value)
	{	IHDSNodeList list=getNodes(value);
		IHDSNodeList result=null;
		if(list!=null)
		{	ArrayList<IHDSNode> valid=new ArrayList();
			for(int i=0; i<list.size(); i++)
			{	IHDSNode n=list.get(i);
				if(HDSUtils.isDescendantOrSelf(ancestor, n))
				{	valid.add(n);				
				}
			}
			if(valid.size()>0)
			{	result=new HDSNodeListImpl(valid);			
			}
		}
		return result;
	}
	
	public IHDSNodeList getNodesWithAncestor(IHDSNode ancestor, IHDSValueMatcher matcher)
	{	IHDSNodeList list=getNodes(matcher);
		IHDSNodeList result=null;
		if(list!=null)
		{	ArrayList<IHDSNode> valid=new ArrayList();
			for(int i=0; i<list.size(); i++)
			{	IHDSNode n=list.get(i);
				if(HDSUtils.isDescendantOrSelf(ancestor, n))
				{	valid.add(n);				
				}
			}
			if(valid.size()>0)
			{	result=new HDSNodeListImpl(valid);			
			}
		}
		return result;
	}
	
	public IHDSNode getRoot()
	{	return mRoot;		
	}
	
	private void mapNode(IHDSNode node)
	{	Object value=node.getValue();
		List<IHDSNode> list=mMap.get(value);
		if(list==null)
		{	list=new ArrayList<IHDSNode>();			
		}
		list.add(node);
		mMap.put(value, list);
	}
	
}
