package org.netkernel.layer0.urii;

import java.util.List;
import org.netkernel.layer0.nkf.INKFExpiryFunction;
import org.netkernel.request.IExpiry;

public class AggregatingPolledExpiry implements INKFExpiryFunction
{
	private final IExpiry[] mExpiries;
	private final long mPoll;
	private final String mDebug;
	private long mNextCheck=1;
	
	public AggregatingPolledExpiry(List<IExpiry> aExpiries, long aPoll, String aDebug)
	{	mExpiries=new IExpiry[aExpiries.size()];
		for (int i=0; i<mExpiries.length; i++)
		{	mExpiries[i]=aExpiries.get(i);
		}
		mPoll=aPoll;
		mDebug=aDebug;
	}

	public boolean isExpired(long aNow)
	{	boolean result;
		if (aNow<mNextCheck)
		{	//too soon to test
			result=false;
		}
		else if (mNextCheck==0)
		{	//once expired always expired
			result=true;
		}
		else if (mNextCheck==1)
		{	//initial state, wait for poll period before testing
			mNextCheck=aNow+mPoll;
			result=false;
		}
		else
		{	result=false;
			for (int i=mExpiries.length-1; i>=0 && !result; i--)
			{	result=mExpiries[i].isExpired(aNow);
			}
			if (result)
			{	//once expired always expired
				mNextCheck=0;
			}
			else
			{	mNextCheck=aNow+mPoll;
			}
		}
		return result;
	}
	
	public String toString()
	{	return mDebug;
	}
}
