/******************************************************************************
  (c) Copyright 2002-2006, 1060 Research Ltd                                   

  This Software is licensed to You, the licensee, for use under the terms of   
  the 1060 Public License v1.0. Please read and agree to the 1060 Public       
  License v1.0 [www.1060research.com/license] before using or redistributing   
  this software.                                                               

  In summary the 1060 Public license has the following conditions.             
  A. You may use the Software free of charge provided you agree to the terms   
  laid out in the 1060 Public License v1.0                                     
  B. You are only permitted to use the Software with components or applications
  that provide you with OSI Certified Open Source Code [www.opensource.org], or
  for which licensing has been approved by 1060 Research Limited.              
  You may write your own software for execution by this Software provided any  
  distribution of your software with this Software complies with terms set out 
  in section 2 of the 1060 Public License v1.0                                 
  C. You may redistribute the Software provided you comply with the terms of   
  the 1060 Public License v1.0 and that no warranty is implied or given.       
  D. If you find you are unable to comply with this license you may seek to    
  obtain an alternative license from 1060 Research Limited by contacting       
  license@1060research.com or by visiting www.1060research.com                 

  NO WARRANTY:  THIS SOFTWARE IS NOT COVERED BY ANY WARRANTY. SEE 1060 PUBLIC  
  LICENSE V1.0 FOR DETAILS                                                     

  THIS COPYRIGHT NOTICE IS *NOT* THE 1060 PUBLIC LICENSE v1.0. PLEASE READ     
  THE DISTRIBUTED 1060_Public_License.txt OR www.1060research.com/license      

  File:          $RCSfile: RequestScopeClassLoader.java,v $
  Version:       $Name:  $ $Revision: 1.8 $
  Last Modified: $Date: 2007/12/13 14:31:13 $
 *****************************************************************************/
package org.netkernel.layer0.util;

import java.util.*;

import org.netkernel.request.IRequestScopeLevel;
import org.netkernel.urii.ClassLoaderWithExports;
/**
 * A Classloader that will load classes from a request scope by accending the
 * request scope of a request to find classes in calling spaces. It also
 * contains a cache which improves performance with some of the liberal
 * ways that classloaders are used in scripting engines. This includes caching
 * of not found exceptions. This classloader adds an overhead as compared to a
 * simple classloader however it gives classloading which is strictly according
 * to the same model as used in resource resolution- this however is rarely needed.
 * @author  tab
 */
public class RequestScopeClassLoader extends ClassLoader
{
	private static final Object CLASSNOTFOUND = new Integer(0);
	
	private IRequestScopeLevel mScope;
	private Map mClassCache = Collections.synchronizedMap(new HashMap());
	
	/** Creates a new instance of ScriptClassLoader */
	public RequestScopeClassLoader(IRequestScopeLevel aScope)
	{	mScope=aScope;
	}
	
	public void cleanup()
	{	mScope=null;
		mClassCache=null;
	}
	
	public Class findClass(String aName) throws ClassNotFoundException
	{	return loadClass(aName);
	}
	
	public Class loadClass(String aName, boolean aResolve) throws ClassNotFoundException
	{	Class c= loadClass(aName);
		if (aResolve)
		{	resolveClass(c);
		}
		return c;
	}
	
	public Class loadClass(String aName) throws ClassNotFoundException
	{	//System.out.println("Loading "+aName);
		Object cached = mClassCache.get(aName);
		if (cached!=null)
		{	//System.out.println("found in cache ");
			if (cached==CLASSNOTFOUND)
			{	throw new ClassNotFoundException(aName);
			}
			else
			{	return (Class)cached;
			}
		}
		
		IRequestScopeLevel scope=mScope;
		ClassLoaderWithExports firstClassLoader=null;
		while (true)
		{	
			ClassLoaderWithExports cl=null;
			while(scope!=null && cl==null)
			{	cl=scope.getSpace().getClassLoader();
				scope=scope.getParent();
			}
			if (cl==null)
			{	break;
			}
			else
			{	if (firstClassLoader==null)
				{	firstClassLoader=cl;
				}
				try
				{	Class result = cl.loadClassAvoidingParent(aName);
					mClassCache.put(aName,result);
					//System.out.println("found "+aName);
					return result;
				}
				catch (ClassNotFoundException e)
				{	
				}
			}
		}
		//resort to parent classloader
		if (firstClassLoader!=null)
		{
			try
			{	ClassLoader parent = firstClassLoader.getParentClassLoader();
				return  parent.loadClass(aName);
			}
			catch (ClassNotFoundException e2)
			{	mClassCache.put(aName,CLASSNOTFOUND);
				throw e2;
			}
		}
		else
		{	throw new ClassNotFoundException(aName);
		}
	}
	
	/** classloaders are equal if they operate on equivalent request scopes 
	 * 	- i.e. all classloader providing scopes provide the same classloaders in the same order 
	 */
	public boolean equals(Object aOther)
	{	boolean result=false;
		if((aOther instanceof RequestScopeClassLoader))
		{	IRequestScopeLevel p1=mScope;
			IRequestScopeLevel p2=((RequestScopeClassLoader)aOther).mScope;
			result=true;
			while (p1!=null || p2!=null)
			{	ClassLoader cl1=null;
				while(p1!=null && cl1==null)
				{	cl1=p1.getSpace().getClassLoader();
					p1=p1.getParent();
				}
				ClassLoader cl2=null;
				while(p2!=null && cl2==null)
				{	cl2=p2.getSpace().getClassLoader();
					p2=p2.getParent();
				}
				if (cl1!=cl2)
				{	result=false;
					break;
				}
			}
		}
		return result;
	}
	
	public String toString()
	{	StringBuffer sb=new StringBuffer(256);
		sb.append("org.netkernel.layer0.util.RequestScopeClassLoader [");
		/*
		sb.append(mScopeLevel.getURI().toString());
		for (int i=mSuperStack.size()-1; i>=0; i--)
		{	ModuleDefinition md=(ModuleDefinition)mSuperStack.get(i);
			sb.append("->");
			sb.append(md.getURI().toString());
		}
		*/
		sb.append(']');
		return sb.toString();
	}
}